-- Novell DFS ASN1 Schema Definitions
-- Novell Inc.
-- 122 East 1700 South
-- Provo, UT 84606-6194

-- VeRsIoN=0.1
-- CoPyRiGhT=(c) Copyright 1999, Novell, Inc.  All rights reserved

-- Object ID Syntax
------------------------------------------
------------------------------------------
-- Change History:
--
-- Thu, Jan 14, 1999: We do not have any ASN1 IDs assigned to us yet.
-- Thu, Jan 28, 1999: application(1)dfs(10) has been assigned to us.
-- Thu, Aug 26, 1999: application(1)dfs(138) has been assigned to us now.
--                    The previous assignment stands void.
--                    Attribute names and class definitions have been
--                    changed to take into account the review comments
--                    from the Schema review board on Aug 23, 99.
--
--                    DFSCachingMech, DFSVolType and DFSReplMech have been
--                    nuked.
--
-- Auxiliary class is not being used because it mandates NDSv8.
-- Naming Convention: DFS- prefix to all generic DFS attributes and classes.
--                    VLDB- prefix to VLDB attributes and classes.
--
-- How to use this file:
------------------------------------------
------------------------------------------
-- Use the tool ndssch located on SYS:\public to make extensions to the
-- in accordance to the specification in a .sch file.
-- Usage: ndssch <file-name>
--
------------------------------------------
------------------------------------------
--
-- ASN1 Object ID's (will most probably be) Defined as Follows:
--     joint-iso-ccitt(2) country(16) us(840) organization(1)
--     Novell(113719) applications(1) DFS(138)
--                                    DFSAttributeType(4)
--                                    DFSClass(6)
--
------------------------------------------
-- One Additional Digit is added after the attribute or object
-- identifier to identify the release number. This number reflects
-- the release number (of the schema) in which the object or attribute
-- was added to the definition. Ed Reed suggested that keeping it after
-- the attribute/object would allow the flexibility to bump up the
-- revision of each object individually. Currently available schemas treat
-- this as the revision of the schema. I'll probably stick with the current
-- convention.
--
-- The first release would have the ID 1 and count up from there.
-- An example of the first attribute of the first release would be:
-- 2 16 840 1 113719 1 138 4 1 1.
-- The fourth attribute in the third release would be:
-- 2 16 840 1 113719 1 138 4 3 4.
------------------------------------------
-- 
-- Attribute range 1-100 is reserved for this file.
-- Class range 1-10 is reserved for this file.
-- 
------------------------------------------


NDFSSchemaExtensions DEFINITIONS ::=
BEGIN

-- New Attributes (generic DFS)
------------------------------------------
------------------------------------------
"DFS-VLDB-Hosts" ATTRIBUTE ::=
{
  SyntaxID    SYN_DIST_NAME,
  Flags       {DS_SYNC_IMMEDIATE, DS_PUBLIC_READ},
  ASN1ObjID   {2 16 840 1 113719 1 138 4 1 1}
}

"DFS-Volume-Context" ATTRIBUTE ::=
{
  SyntaxID    SYN_DIST_NAME,
  Flags       {DS_SYNC_IMMEDIATE, DS_SINGLE_VALUED_ATTR, DS_PUBLIC_READ},
  ASN1ObjID   {2 16 840 1 113719 1 138 4 1 2}
}

"DFS-Volume-Guid" ATTRIBUTE ::=
{
  SyntaxID    SYN_OCTET_STRING,
  Flags       {DS_SYNC_IMMEDIATE, DS_SINGLE_VALUED_ATTR, DS_PUBLIC_READ},
  ASN1ObjID   {2 16 840 1 113719 1 138 4 1 3}
}

"VLDB-BackEnd-ID" ATTRIBUTE ::=
{
  SyntaxID    SYN_CI_STRING,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_STRING_ATTR, DS_PUBLIC_READ},
  ASN1ObjID   {2 16 840 1 113719 1 138 4 1 7}
}


-- Class Modification (generic DFS)
------------------------------------------
------------------------------------------
"Organization" OBJECT-CLASS ::=
{
    Operation   MODIFY,
    MayContain  {
        "DFS-VLDB-Hosts",
        "VLDB-BackEnd-ID"
    }
}

"Organizational Unit" OBJECT-CLASS ::=
{
    Operation   MODIFY,
    MayContain  {
        "DFS-VLDB-Hosts",
        "VLDB-BackEnd-ID"
    }
}

"Volume" OBJECT-CLASS ::=
{
    Operation   MODIFY,
    MayContain  {"DFS-Volume-Guid"}
}

-- New Classes (generic DFS)
------------------------------------------
------------------------------------------

"DFS-Junction" OBJECT-CLASS ::=
{
    Operation   ADD,
    SubClassOf  {"Top"},
    Flags       {DS_EFFECTIVE_CLASS},
    ContainedBy {"Organization", "Organizational Unit"},
    NamedBy     {"CN"},
    MustContain {
        "CN",
        "DFS-Volume-Guid",
        "DFS-Volume-Context"
    },
    MayContain  {
        "Version"
    },
    ASN1ObjID   {2 16 840 1 113719 1 138 6 1 1}
}

END
