#!/usr/bin/env perl

#
# (C) Copyright 2004-2015, 2018-2020 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and
# licensors (“Micro Focus”) are as may be set forth in the express warranty statements
# accompanying such products and services. Nothing herein should be construed as
# constituting an additional warranty. Micro Focus shall not be liable for technical
# or editorial errors or omissions contained herein. The information contained herein
# is subject to change without notice.
#
# WARNING:  Unauthorized reproduction of this program as well as
# unauthorized preparation of derivative works based upon the
# program or distribution of copies by sale, rental, lease or
# lending are violations of federal copyright laws and state trade
# secret laws, punishable by civil and criminal penalties.
#
#***************************************************************************
#
# Novell Storage Services(NSS) command line utility
#
#---------------------------------------------------------------------------


########################################################################
# Perl script to emulate parts of NetWare's nss command.
########################################################################

# Command switches
# Multiple entries allow any of the specified options to be used
# to specify the command.  This is a bit crufty, and you will have
# to be careful when adding new items.  If you don't like the way
# it is, feel free to fix it so it is to your liking.
@Help=("/h", "/he", "/hel", "/help", "help");
@Pools=("/pools");
@Poolactivate=("/poola", "/poolac", "/poolact", "/poolacti", "/poolactiv",
               "/poolactiva", "/poolactivat", "/poolactivate");
@Pooldeactivate=("/poold", "/poolde", "/pooldea", "/pooldeac", "/pooldeact",
                 "/pooldeacti", "/pooldeactiv", "/pooldeactiva",
                 "/pooldeactivat", "/pooldeactivate");
@Poolmigrate=("/poolmi", "/poolmig", "/poolmigr", "/poolmigra", "/poolmigrat",
	      "/poolmigrate");
@Poolmaintenance=("/poolm", "/poolma", "/poolmai", "/poolmain", "/poolmaint",
                  "/poolmainte", "/poolmainten", "/poolmaintena","/poolmaintenan",
                  "/poolmaintenanc","/poolmaintenance");
@Volumes=("/volumes");
@Volumeactivate=("/volumea", "/volumeac", "/volumeact", "/volumeacti", "/volumeactiv",
               "/volumeactiva", "/volumeactivat", "/volumeactivate");
@Volumedeactivate=("/volumed", "/volumede", "/volumedea", "/volumedeac", "/volumedeact",
                 "/volumedeacti", "/volumedeactiv", "/volumedeactiva",
                 "/volumedeactivat", "/volumedeactivate");
@Error=("/err", "/erro", "/error", "/errorc", "/errorco", "/errorcod", "/errorcode");


{
    my $rc;

    if ($#ARGV < 0 || $ARGV[0] eq "/?" || $ARGV[0] eq "-?")
    {
        DisplayHelp("");
        exit;
    }

    my $done = 0;
    my $switch, $remainder;
    ($switch, $remainder) = split(/=/, lc $ARGV[0], 2);

    HELP: foreach(@Help)
    {
        if ($switch eq $_)
        {
            $rc = DisplayHelp("/help");
            exit $rc;
        }
    }
    
    open(NSSFILE, "+</_admin/Manage_NSS/manage.cmd")
        or die "Error opening NSS management file ($!) on server";
    
	unless ($done)
	{
		ERROR: foreach(@Error)
		{
			if ($switch eq $_)
			{
				$rc = GetErrorName($remainder);
				$done = 1;
				last ERROR;
			}
		}
	}

    unless ($done)
    {
      POOLS: foreach(@Pools)
      {
          if ($switch eq $_)
          {
              $rc = DisplayPools();
              $done = 1;
              last POOLS;
          }
      }
    }

    unless ($done)
    {
      POOLACTIVATE: foreach(@Poolactivate)
      {
          if ($switch eq $_)
          {
              $rc = ActivatePool($remainder, "active");
              $done = 1;
              last POOLACTIVATE;
          }
      }
    }

    unless ($done)
    {
      POOLMIGRATE: foreach(@Poolmigrate)
      {
          if ($switch eq $_)
          {
              $rc = MigratePool($remainder);
              $done = 1;
              last POOLMIGRATE;
          }
      }
    }

    unless ($done)
    {
      POOLDEACTIVATE: foreach(@Pooldeactivate)
      {
          if ($switch eq $_)
          {
              $rc = DeactivatePool($remainder);
              $done = 1;
              last POOLDEACTIVATE;
          }
      }
    }
    
    unless ($done)
    {
      POOLMAINTENANCE: foreach(@Poolmaintenance)
      {
          if ($switch eq $_)
          {
              $rc = ActivatePool($remainder,"maintenance");
              $done = 1;
              last POOLMAINTENANCE;
          }
      }
    }

    unless ($done)
    {
      VOLUMES: foreach(@Volumes)
      {
          if ($switch eq $_)
          {
              $rc = DisplayVolumes($remainder);
              $done = 1;
              last VOLUMES;
          }
      }
    }

    unless ($done)
    {
      VOLUMEACTIVATE: foreach(@Volumeactivate)
      {
          if ($switch eq $_)
          {
              $rc = ActivateVolume($remainder);
              $done = 1;
              last VOLUMEACTIVATE;
          }
      }
    }

    unless ($done)
    {
      VOLUMEDEACTIVATE: foreach(@Volumedeactivate)
      {
          if ($switch eq $_)
          {
              $rc = DeactivateVolume($remainder);
              $done = 1;
              last VOLUMEDEACTIVATE;
          }
      }
    }

    unless ($done)
    {
        my $arg = join(" ", @ARGV);
        $rc = CallNSSCLI($arg);
    }

    close(NSSFILE);
    exit $rc;
}


########################################################################
# Process nss /help command
########################################################################
sub DisplayHelp($)
{
#    print "Available options:\n".
#          "  nss /help                      	Display this message\n".
#          "  nss /pools                     	List NSS pools\n".
#          "  nss /poolactivate=<PoolName>   	Activate an NSS pool\n".
#          "  nss /poolmaintenance=<PoolName>	Place an NSS pool in Maintenance mode\n".
#          "  nss /pooldeactivate=<PoolName> 	Deactivate an NSS pool\n".
#          "  nss /volumes                    	List NSS Volumes\n".
#          "  nss /volumeactivate=<VolumeName>	Activate an NSS Volume\n".
#          "  nss /volumedeactivate=<VolumeName>	Deactivate an NSS Volume\n".
#          "  nss /errorcode=<Error code>		Translate error code to a string\n".
#          "\n";
    CallNSSCLI($_[0]);

    return 0;
}


sub GetErrorName($)
{
    my $ret;
    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
					"<nssRequest><server><getErrorName>".
					"<errorCode>".$_[0]."</errorCode>".
					"</getErrorName></server></nssRequest>");
	$ret = DisplayErrorName($nssResult);
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }
    return $ret;
}

########################################################################
# Process nss /pools command
########################################################################
sub DisplayPools()
{
    my $ret;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><server><listPools/>".
                                     "</server></nssRequest>");
        $ret = ParseListPools($nssResult);
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }

    return $ret;
}

########################################################################
# Process nss /poolactivate or /poolmaintenance command
########################################################################

sub ActivatePool($)
{
    my $result;
    my $ret;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><pool><modifyState>".
                                     "<poolName>".$_[0]."</poolName>".
                                     "<poolState>".$_[1]."</poolState>".
                                     "</modifyState></pool></nssRequest>");
        $result .= GetResultCode($nssResult, "modifyState");
        if ($result eq 20812)
	{
	    system '/sbin/nlvm rescan';	
	    $nssResult = WriteCommand(NSSFILE, 
				      "<nssRequest><pool><modifyState>".
				      "<poolName>".$_[0]."</poolName>".
				      "<poolState>".$_[1]."</poolState>".
				      "</modifyState></pool></nssRequest>");
	}
	$result .= ParseBasicReply($nssResult, "modifyState");
    }
    else
    {
        print "Can't send XML request: $ret\n";
        $result = $ret;
    }

    return $result;
}

########################################################################
# Process nss /pooldeactivate command
########################################################################
sub DeactivatePool($)
{
    my $result;
    my $ret;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
	my $nssResult;
	$ppid = getppid();
	$name = `ps -o args -p $ppid | grep -v COMMAND`;
	chomp($name);
	if ($name =~ /unload/)
	{
		$nssResult = WriteCommand(NSSFILE, 
				"<nssRequest><pool><modifyState>".
				"<poolName>".$_[0]."</poolName>".
				"<poolState>deactive</poolState>".
				"<migrate>true</migrate>".
				"</modifyState></pool></nssRequest>");
	}
	else
	{
		$nssResult = WriteCommand(NSSFILE, 
				"<nssRequest><pool><modifyState>".
				"<poolName>".$_[0]."</poolName>".
				"<poolState>deactive</poolState>".
				"</modifyState></pool></nssRequest>");


	}
        $result .= ParseBasicReply($nssResult, "modifyState");
    }
    else
    {
        print "Can't send XML request: $ret\n";
        $result = $ret;
    }

    return $result;
}

########################################################################
# Process nss /poolmigrate command
########################################################################
sub MigratePool($)
{
    my $result;
    my $ret;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><pool><modifyState>".
                                     "<poolName>".$_[0]."</poolName>".
                                     "<poolState>deactive</poolState>".
				     "<migrate>true</migrate>".	
                                     "</modifyState></pool></nssRequest>");
        $result .= ParseBasicReply($nssResult, "modifyState");
    }
    else
    {
        print "Can't send XML request: $ret\n";
        $result = $ret;
    }

    return $result;
}
########################################################################
# Process nss /volumes command
########################################################################
sub DisplayVolumes()
{
    my $ret;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><volume>".
				     "<listVolumes type=\"all\"></listVolumes>".
                                     "</volume></nssRequest>");
        $ret = ParseListVolumes($nssResult);
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }

    return $ret;
}

########################################################################
# Process nss /volumeactivate command
########################################################################
sub ActivateVolume($)
{
    my $result;
    my $ret;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><volume><modifyState>".
                                     "<volumeName>".$_[0]."</volumeName>".
                                     "<volumeState>mounted</volumeState>".
                                     "</modifyState></volume></nssRequest>");
        $result .= ParseBasicReply($nssResult, "modifyState");
    }
    else
    {
        print "Can't send XML request: $ret\n";
        $result = $ret;
    }

    return $result;
}

########################################################################
# Process nss /volumedeactivate command
########################################################################
sub DeactivateVolume($)
{
    my $result;
    my $ret;
    my $ppid;
    my $name;
    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
	my $nssResult;
	$ppid = getppid();
	$name = `ps -o args -p $ppid | grep -v COMMAND`;
	chomp($name);
	if ($name =~ /dismount/)
	{
		$nssResult = WriteCommand(NSSFILE, 
				"<nssRequest><volume><modifyState>".
				"<volumeName>".$_[0]."</volumeName>".
				"<volumeState>deactive</volumeState>".
				"<migrate>true</migrate>".
				"</modifyState></volume></nssRequest>");
	}
	else 
	{
		$nssResult = WriteCommand(NSSFILE, 
				"<nssRequest><volume><modifyState>".
				"<volumeName>".$_[0]."</volumeName>".
				"<volumeState>deactive</volumeState>".
				"</modifyState></volume></nssRequest>");
	}
	$result .= ParseBasicReply($nssResult, "modifyState");
    }
    else
    {
        print "Can't send XML request: $ret\n";
        $result = $ret;
    }

    return $result;
}


########################################################################
# Process other nsscon command
########################################################################
sub CallNSSCLI($)
{
    print `/opt/novell/nss/sbin/nsscli $_[0]`;
    return 0;
}

########################################################################
# Get the attribute & volume infor for the specified pool
########################################################################
sub GetPoolInfo($)
{
    my $result;
    my $ret;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><pool><getPoolInfo>".
                                     "<poolName>".$_[0]."</poolName>".
                                     "<attributes/>".
                                     "<volumes/>".
                                     "</getPoolInfo></pool></nssRequest>");
        $nssResult =~ /<getPoolInfo>(.*?)<\/getPoolInfo>/gs;
        $result = $1;
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }

    return $result;
}

########################################################################
# Get the attribute information for the specified volume
########################################################################
sub GetVolumeInfo($)
{
    my $result;
    my $ret;

    if (!($ret = SetDataStream(NSSFILE, "command")))
    {
        my $nssResult = WriteCommand(NSSFILE, 
                                     "<nssRequest><volume><getVolumeInfo type=\"basic\">".
                                     "<volumeName>".$_[0]."</volumeName>".
				     "<basic/>".
                                     "<attributes/>".
                                     "</getVolumeInfo></volume></nssRequest>");
        $nssResult =~ /<getVolumeInfo>(.*?)<\/getVolumeInfo>/gs;
        $result = $1;
    }
    else
    {
        print "Can't send XML request: $ret\n";
    }

    return $result;
}

########################################################################
# Get the attributes for the specified pool
########################################################################
sub GetPoolAttributes($)
{
	my @temp;
	my @result;
	my $response = $_[0];

    $response =~ /<enabledAttributes.*?>(.*?)<\/enabledAttributes>/gs;
    @temp = $1 =~ /<(.*?)>/gs;
	foreach $item (@temp)
	{
		if ($item =~ m/^\/(.*?)/)
		{
			push @result, $';
		}

		$x = substr $item, -1;
		if ($x == "\/")
		{
			$y = substr $item, 0, -1;
			push @result, $y;
		}
	}
	return @result;
}

########################################################################
# Get the volumes for the specified pool
########################################################################
sub GetPoolVolumes($)
{
    my @result;
    my $response = $_[0];

    $response =~ /<volumeInfo>(.*?)<\/volumeInfo>/gs;
    @result = $1 =~ /<volumeName>(.*?)<\/volumeName>/gs;

    return @result;
}

########################################################################
# Parse the result of the listVolumes operation
########################################################################
sub ParseListVolumes($)
{
    my $xml = $_[0];
    my $result;
    my @xml;
    my $header;
    my $out;
    my @volumes;

    @xml = $xml =~ /<listVolumes>(.*?)<\/listVolumes>/gs;
    $header =  "     Volume Name          State         Attributes   \n";
    $header .= "----------------------  -----------  ----------------- \n";
    $out = "";

    if ($result =~ /<result value=\"(-?\d+)\">/s)
    {
    	if ($1 != 0)
        {
	    print "Error $1 getting pool list\n";
            return $1;
        }
    }

    @volumes = $xml =~ /<volumeName type=\"(.*?)\">(.*?)<\/volumeName>/gs;
    while(@volumes) 
    {
	# Get Volume Info
	$volumeInfo = GetVolumeInfo(@volumes[1]);

	# Get Volume State	
        if ($volumeInfo =~ /<volumeState>(.*?)<\/volumeState>/s)
	{
            $volumeState = $1;
	}
	if ($volumeState eq "mounted")
	{
    		$out .= sprintf("%-22.17s  %-17.17s", @volumes[1], uc "active");
	}
	else
	{ 
    		$out .= sprintf("%-22.17s  %-17.17s", @volumes[1], uc $volumeState);
	}

	# Get attribute info from response
	@attrs =  GetPoolAttributes($volumeInfo);

        $attrFound = 0;
        my $first = 1;

        LISTATTRS: while (@attrs)
        {
            $attr = lc @attrs[0];
            if ($attr eq lc "volumeHardlinks")
            {
                $attrText = "Hardlinks";
            }
            elsif ($attr eq lc "volumeEncrypted")
            {
                $attrText = "Encrypted";

                if (@attrs)
                {
                    $shiftedAttr = shift @attrs;
                    $attr = lc @attrs[0];

                    if ($attr eq lc "AES128")
                    {
                        $attrText .= "[AES128]";
                    }
                    elsif ($attr eq lc "AES256")
                    {
                        $attrText .= "[AES256]";
                    }
                    else
                    {
                        unshift(@attrs, $shiftedAttr);
                    }
                }
            }
            elsif ($attr eq lc "salvage")
            {
                $attrText = "Salvage";
            }
            elsif ($attr eq lc "compression")
            {
                $attrText = "Compression";
            }
            elsif ($attr eq lc "directoryQuota")
            {
                $attrText = "Directory Quotas";
            }
            elsif ($attr eq lc "userQuota")
            {
                $attrText = "User Quotas";
            }
            elsif ($attr eq lc "flushFiles")
            {
                $attrText = "Immediately Flush Files On Close";
            }
            elsif ($attr eq lc "cfsMaster")
            {
                $attrText = "Cluster File System Master";
            }
            elsif ($attr eq lc "mfl")
            {
                $attrText = "Modified File List";
            }
            elsif ($attr eq lc "snapshot")
            {
                $attrText = "Snapshot";
            }
            elsif ($attr eq lc "shredding")
            {
                $attrText = "Data Shredding";
            }
            elsif ($attr eq lc "userTransaction")
            {
                $attrText = "User Transactions";
            }
            elsif ($attr eq lc "migration")
            {
                $attrText = "Migration";
            }
            elsif ($attr eq lc "noAccessTime")
            {
                $attrText = "No Access Time";
            }
            elsif ($attr eq lc "volumeADEnabled")
            {
                $attrText = "AD Enabled";
            }
            elsif ($attr eq lc "TrusteeIndex")
            {
                $attrText = "Trustee Index";
            }
            else
            {
                if (@attrs)
                {
                    next LISTATTRS;
                }
                else
                {
                    $attrText = "";
                }
            }
            $attrFound = 1;
            if ($first)
            {
                $first = 0;
                $out .= sprintf("  %-17.17s  \n", $attrText);
            }
            else
            {
                $out .= sprintf("%-22.17s  %17.17s  %-17.17s\n",
                                "", "", $attrText);
            }
        }
        continue
        {
            shift @attrs;
        }
        $out .= "\n" unless $attrFound;
    
        shift @volumes;
        shift @volumes;
    }

    print $header;
    my @lines = split /\n(?=[A-Z,0-9,_,!,@,#,$,%,&,(,)])/, $out;
    my @sorted_lines = sort @lines;
    print join "\n", @sorted_lines;
    print "\n";

    return 0;
}


########################################################################
# Parse the result of the list partition operation
########################################################################
sub ParseListPools($)
{
    my $xml = $_[0];
    my $result;
    my @xml;
    my @attrs;
    my @volumes;
    my $attrText;
    my $attrFound;
    my $header;
    my $out;
    my $poolName;
    my $poolState;

#       print "Result xml=$xml\n";
    @xml = $xml =~ /<poolSimpleInfo>(.*?)<\/poolSimpleInfo>/gs;
    $header =  "    Pool Name         State        Attributes       Logical Volumes\n";
    $header .= "-----------------  -----------  -----------------  -------------------\n";
    $out = "";

    foreach $result (@xml)
    {
        if ($result =~ /<result value=\"(-?\d+)\">/s)
        {
            if ($1 != 0)
            {
                print "Error $1 getting pool list\n";
                return $1;
            }
        }

        $poolName = "";
        $poolState = "";
        if ($result =~ /<poolName>(.*?)<\/poolName>/s)
        {
            $poolName = $1;
        }
        if ($result =~ /<poolState>(.*?)<\/poolState>/s)
        {
            $poolState = $1;
        }
        # Get Pool Info
        $poolInfo = GetPoolInfo($poolName);
        # Get attribute info from response
        @attrs = GetPoolAttributes($poolInfo);
        # Get volume info from response
        @volumes = GetPoolVolumes($poolInfo);

        $out .= sprintf("%-17.17s  %-11.11s", $poolName, uc $poolState);
        $attrFound = 0;
        my $first = 1;
        LISTATTRS: while (@attrs || @volumes)
        {
            $attr = lc @attrs[0];
            if ($attr eq lc "shared")
            {
                $attrText = "Shared (Clustered)";
            }
            elsif ($attr eq lc "readOnly")
            {
                $attrText = "Read-Only";
            }
            elsif ($attr eq lc "multipleVolumes")
            {
                $attrText = "Multiple Volumes";
            }
            elsif ($attr eq lc "snapshot")
            {
                $attrText = "Pool Snapshot";
            }
            elsif ($attr eq lc "msap")
            {
                $attrText = "Multi use detect";
            }
            elsif ($attr eq lc "autoDiscard")
            {
                $attrText = "Auto Discard";
            }
            else
            {
                if (@attrs)
                {
                    next LISTATTRS;
                }
                else
                {
                    $attrText = "";
                }
            }
            $attrFound = 1;
            if ($first)
            {
                $first = 0;
                $out .= sprintf("  %-17.17s  %-17.17s\n", $attrText,@volumes[0]);
            }
            else
            {
                $out .= sprintf("%-17.17s  %-11.11s  %-17.17s  %-17.17s\n",
                                "", "", $attrText, @volumes[0]);
            }
            shift @volumes;
        }
        continue
        {
            shift @attrs;
        }
        $out .= "\n" unless $attrFound;
    }

    print $header;
    my @lines = split /\n(?=[A-Z,0-9,_,!,@,#,$,%,&,(,)])/, $out;
    my @sorted_lines = sort @lines;
    print join "\n", @sorted_lines;
    print "\n";

    return 0;
}

########################################################################
# Parse the result of a basic reply (just OK/error)
########################################################################
sub ParseBasicReply($$)
{
    my $xml = $_[0];
    my $request = $_[1];
    my $result;
    my $rc;
    my @xml;

    @xml = $xml =~ /<$request>(.*?)<\/$request>/gs;
    foreach $result (@xml)
    {
        if ($result =~ /<result value=\"(-?\d+)\">/s)
        {
            $rc = $1;
            if ($1 != 0)
            {
                print "Error $1\n";
            }
        }
        if ($result =~ /<description>(.*?)<\/description>/s)
        {
            if ($1 ne "success")
            {
                print "$1\n";
            }
        }
    }
    return $rc;
}

########################################################################
# Parse the result of a basic reply (just OK/error)
########################################################################
sub GetResultCode($$)
{
    my $xml = $_[0];
    my $request = $_[1];
    my $result;
    my $rc;
    my @xml;

    @xml = $xml =~ /<$request>(.*?)<\/$request>/gs;
    foreach $result (@xml)
    {
        if ($result =~ /<result value=\"(-?\d+)\">/s)
        {
            $rc = $1;
        }
    }
    return $rc;
}


########################################################################
# Read error code from /dev/nsscmd
########################################################################
sub GetErrorCode($$)
{
	my $errorcode = $_[0];
	my $DEVNSSCMD;
	my $command = "/error=$errorcode";
	my $reply = "";
	my $tmpReply;
	open ($DEVNSSCMD, "+</dev/nsscmd")
	or die "Error opening /dev/nsscmd file ($!) on server";

	#Send error code to /dev/nsscmd
	if (!syswrite ($DEVNSSCMD, $command, length ($command)))
	{
		print "Error : Unable to write to /dev/nsscmdi ($!)\n";
		return -1;
    	}
	else
	{
		my @tmpRead1;
		my @tmpRead2;
		$reply = "";
		#Read error description from /dev/nsscmd
		READLOOP: while ( $ret = sysread($DEVNSSCMD, $tmpReply, 100))
		{
			if (!defined($ret))
			{
				#Error is reading /dev/nsscmd
				print "Error : Unable to read error code description ($!).\n";
				return -2;
			}
		 	$reply .= $tmpReply;
			if ($ret < 100)
			{
				#Nothing more to read for now.
				last READLOOP;
			}
		}
		# If output has error description for multiple codes,
		# read last error descritpion.
		if (length($reply) == 0)
		{
			#Error is reading /dev/nsscmd
			print "Error : Unable to read error code description ($!).\n";
			return -3;
		}
		@tmpRead1 = split 'Error=', $reply;
		if ($#tmpRead1 < 0)
		{
			#Error is reading /dev/nsscmd
			print "Error : Unable to read error code description ($!).\n";
			return -4;
		}
		@tmpRead2 = split '\n', $tmpRead1[$#tmpRead1];
		if ($#tmpRead2 < 0)
		{
			#Error is reading /dev/nsscmd
			print "Error : Unable to read error code description ($!).\n";
			return -5;
		}
		$reply = $tmpRead2[0];
		print "Error=".$reply."\n";
	}
	close ($DEVNSSCMD);
	return 0;
}

########################################################################
# Set the datastream for a file handle to the passed in value.
########################################################################
sub SetDataStream(*$)
{
    my $fh = $_[0];
    my $dataStream = $_[1];

    my $result;
    my $command;

    $command = "<virtualIO><datastream name=\"$dataStream\"/></virtualIO>";
    seek $fh, 0, 0;
    if (!syswrite($fh, $command, length($command)))
    {
        $result .= "Unable to send datastream command to NDS management.  ";
        seek $fh, 0, 0;
        if (sysread($fh, $error, 3000)) 
        {
            $result .= $error;
        }
        $result .= "\n";
    }
    return $result;
}

########################################################################
# Parse the result of the getErrorDescription operation
########################################################################
sub DisplayErrorName($)
{
	my $xml = $_[0];
	my @errorName;
	my @name;
	my @desc;
	my @errorCode;
	@errorName = $xml =~ /<getErrorName>(.*?)<\/getErrorName>/gs;
	if (@errorName)
	{
		@name = $errorName[0] =~ /<name>(.*?)<\/name>/gs;
		if (@name)
		{
			@errorCode = $errorName[0] =~ /<errorCode>(.*?)<\/errorCode>/gs;
          	if (@errorCode)
			{
				print "Error=$name[0]\n";
				return 0;
			}
			else
           	{
				@desc = $errorName[0] =~ /<description>(.*?)<\/description>/gs;
				print "Request failed : $desc[0]\n";
				return 0;
			}
		}
	}
	else
	{
		@errorName = $xml =~ /<writeResult>(.*?)<\/writeResult>/s;
		if (@errorName)
		{
			@name = $errorName[0] =~ /<error>(.*?)<\/error>/gs;
			if (@name)
			{
				print "Request failed : $name[0]\n";
				return 0;
			}
		}
	}
	print "Request failed\n$xml\n";
	return -1;
}
########################################################################
# Write a command to a file and get the result
########################################################################
sub WriteCommand(*$)
{
    my $fh = $_[0];
    my $command = $_[1];

    my $result;
    my $ret;

#       print("command=$command\n");
    seek $fh, 0, 0;
    if (!syswrite($fh, $command, length($command)))
    {
        $result .= "Unable to send command to virtual file.  ";
        seek $fh, 0, 0;
        while ($ret = sysread($fh, $errorReply, 3000)) 
        {
            $result .= $errorReply;
            if ($errorReply =~ m/<error>.*<\/error>/gs)
            {
                last;
            }
	    }
        $result .= "\n";
    }
    else
    {
        seek $fh, 0, 0;
        while ( $ret = sysread($fh, $reply, 3000))
		{
            $result .= $reply;
            if ($reply =~  m/<error>.*<\/error>/gs)
            {
                last;
            }
        }
    }
#       print("result=$result\n");
    return $result;
}

