-- Novell NSSFS ASN1 Schema Definitions
-- Novell Inc.
-- 1800 South Novell Place
-- Provo, UT 84606-6194
--

--
-- VeRsIoN=0.1
--
-- CoPyRiGhT=(c) Copyright 2000, Novell, Inc.  All rights reserved
--

-- 
-- Object ID Syntax
------------------------------------------
------------------------------------------
-- Change History:
--
-- Wed, Nov 1, 2000: Incorporate changes suggested by Schema Review
-- Board, use officially-assigned OID values
-- Mon, Oct 2, 2000: Initial definition using DFS OIDs - these will
-- have to change.
--
-- Auxiliary class is not being used because it mandates NDSv8.
--
--
------------------------------------------
------------------------------------------
-- ASN1 Object ID's Defined as Follows:
--     joint-iso-ccitt(2) country(16) us(840) organization(1)
--     Novell(113719) applications(1) NSSFS(219)
--                                    NSSFSAttributeType(4)
--                                    NSSFSClass(6)
--
------------------------------------------
-- 
-- Attribute range 1-100 is reserved for this file.
-- Class range 1-10 is reserved for this file.
-- 
------------------------------------------

NSSFSSchemaExtensions DEFINITIONS ::=
BEGIN

-- New Attributes
------------------------------------------
------------------------------------------
--
-- The nssfsPool attribute is used to identify which Pool a Volume is
-- contained in.
"nssfsPool" ATTRIBUTE ::=
{
  SyntaxID    SYN_DIST_NAME,
  Flags       {DS_SYNC_IMMEDIATE, DS_SINGLE_VALUED_ATTR},
  ASN1ObjID   {2 16 840 1 113719 1 219 4 1}
}

-- The nssfsPoolID attribute holds the pool's GUID
"nssfsPoolID" ATTRIBUTE ::=
{
  SyntaxID   SYN_OCTET_STRING,
  Flags      {DS_SYNC_IMMEDIATE, DS_SINGLE_VALUED_ATTR},
  ASN1ObjID  {2 16 840 1 113719 1 219 4 2}
}

-- The nssfsShared attribute specifies whether the media that contains the
-- pool is attached to multiple servers (e.g., via a SAN), or is directly
-- attached to the specified server.
"nssfsShared" ATTRIBUTE ::=
{
  SyntaxID    SYN_BOOLEAN,
  Flags       {DS_SYNC_IMMEDIATE, DS_SINGLE_VALUED_ATTR},
  ASN1ObjID   {2 16 840 1 113719 1 219 4 3}
}

-- The linuxNCPMountPoint attribute holds the volumes linux mount point.
"linuxNCPMountPoint" ATTRIBUTE ::=
{
  Operation	ADD,
  Flags		{DS_SYNC_IMMEDIATE, DS_SIZED_ATTR},
  SyntaxID	SYN_CI_STRING,
  LowerBound	1,
  UpperBound	4096,
  ASN1ObjID	{2 16 840 1 113719 1 451 4 1 1}
}

-- Class Modification
------------------------------------------
------------------------------------------

-- NW6 NSS Volumes are contained in Pools.  This link is added for the benefit
-- of management tools and Clustering Services.

"Volume" OBJECT-CLASS ::=
{
    Operation   MODIFY,
    MayContain  {"nssfsPool"}
}

-- The linux mount point is needed for migration
"Volume" OBJECT-CLASS ::=
{
   Operation	MODIFY,
   MayContain	{"linuxNCPMountPoint"}
}

-- New Classes
------------------------------------------
------------------------------------------

-- Pools represent the storage that contains Volumes.  There can be
-- many (perhaps thousands) volumes in a single pool.
-- If there is no nssfsShared attribute, a value of FALSE is assumed.
"nssfsPool" OBJECT-CLASS ::=
{
    Operation   ADD,
    SubClassOf  {"Resource"},
    Flags       {DS_EFFECTIVE_CLASS},
    MustContain {
        "Host Server",
        "Host Resource Name",
        "nssfsPoolID"
},
    MayContain  {
        "Status",
        "nssfsShared"
    },
    ASN1ObjID   {2 16 840 1 113719 1 219 6 1}
}

END

